@@ -1,38 +1,24 @@
 #
-# DESCRIPTION
+# DESCRIPTION:
 #   PerlORM - Object relational mapper (ORM) for Perl. PerlORM is Perl
 #   library that implements object-relational mapping. Its features are
 #   much similar to those of Java's Hibernate library, but interface is
 #   much different and easier to use.
 #
-# AUTHOR
+# AUTHOR:
 #   Alexey V. Akimov <akimov_alexey@sourceforge.net>
 #
 # COPYRIGHT
-#   Copyright (C) 2005-2006 Alexey V. Akimov
-#
-#   This library is free software; you can redistribute it and/or
-#   modify it under the terms of the GNU Lesser General Public
-#   License as published by the Free Software Foundation; either
-#   version 2.1 of the License, or (at your option) any later version.
-#   
-#   This library is distributed in the hope that it will be useful,
-#   but WITHOUT ANY WARRANTY; without even the implied warranty of
-#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#   Lesser General Public License for more details.
-#   
-#   You should have received a copy of the GNU Lesser General Public
-#   License along with this library; if not, write to the Free Software
-#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+#   Copyright (c) 2005 Alexey V. Akimov. All rights reserved.
 #
 
 package ORM;
 
-use 5.006001;
+use 5.008001;
 use strict;
 use warnings;
 use Carp;
-use base 'Class::Data::Inheritable';
+use Scalar::Util 'weaken';
 
 use ORM::Error;
 use ORM::Cache;
@@ -49,15 +35,18 @@ use ORM::MetapropBuilder;
 use ORM::ResultSet;
 use ORM::StatResultSet;
 
-our $VERSION = 0.83;
+our $VERSION  = 0.8;
 
-ORM->mk_classdata( '_class_hier' );
-ORM->mk_classdata( '_db' );
-ORM->mk_classdata( '_history_class' );
-ORM->mk_classdata( '_default_prefer_lazy_load' );
-ORM->mk_classdata( '_emulate_foreign_keys' );
-ORM->mk_classdata( '_default_cache_size' );
-ORM->mk_classdata( '_current_transaction' );
+my $initialized;
+my $db;
+my $ta;
+
+my $history_class;
+my $prefer_lazy_load;
+my $emulate_foreign_keys;
+my $default_cache_size;
+
+my %hier = ();
 
 ##
 ## CONSTRUCTORS
@@ -94,7 +83,6 @@ sub new
     my $class   = shift;
     my %arg     = @_;
     my $error   = ORM::Error->new;
-    my $ta      = $class->new_transaction( error=>$error );
     my $self    = {};
     my $history = defined $arg{history} ? $arg{history} : $class->history_is_enabled;
 
@@ -109,8 +97,6 @@ sub new
 
         bless $self, $class;
 
-        $self->{_ORM_tpm} = 1 if( $arg{temporary} );
-
         # Extract required DB properties from %arg
         for $prop ( $class->_not_mandatory_props )
         {
@@ -131,12 +117,12 @@ sub new
     unless( $error->fatal )
     {
         # Check validity of object data
-        $self->_validate_prop( prop=>$self->{_ORM_data}, method=>'new', error=>$error );
+        $self->_validate_prop( prop=>$self->{_ORM_data}, error=>$error );
     }
 
     if( ! $arg{temporary} && ! $error->fatal )
     {
-        $self->{_ORM_data}{id} = $class->_db->insert_object
+        $self->{_ORM_data}{id} = $db->insert_object
         (
             id     => $arg{repair_id},
             object => $self,
@@ -150,14 +136,13 @@ sub new
         # Make record in history
         if( !$error->fatal && $history )
         {
-            $class->_history_class->new( obj=>$self, created=>1, error=>$error );
+            $history_class->new( obj=>$self, created=>1, error=>$error );
+            $self->delete( error=>$error, history=>0 ) if( $error->fatal );
         }
-
-        # Cache object
-        $self->_cache->add( $self ) unless( $error->fatal );
     }
 
-    $error->upto( $arg{error} );
+    $self->_cache->add( $self ) unless( $error->fatal );
+    $arg{error} && $arg{error}->add( error=>$error );
     return $error->fatal ? undef : $self;
 }
 
@@ -170,7 +155,7 @@ sub new
 sub count
 {
     my $class = shift;
-    $class->_db->count( class=>$class, @_ );
+    $db->count( class=>$class, @_ );
 }
 
 sub exists
@@ -238,7 +223,7 @@ sub find
         $res = ORM::ResultSet->new
         (
             class  => $class,
-            result => $class->_db->select_base
+            result => $db->select_base
             (
                 class    => $class,
                 filter   => $arg{filter},
@@ -256,7 +241,7 @@ sub find
     }
     else
     {
-        $res = $class->_db->select_full
+        $res = $db->select_full
         (
             class    => $class,
             filter   => $arg{filter},
@@ -286,7 +271,7 @@ sub find
         }
     }
 
-    $error->upto( $arg{error} );
+    $arg{error} && $arg{error}->add( error=>$error );
     
     return
         $arg{return_res}
@@ -301,6 +286,28 @@ sub find
 ##     error     => ORM::Error,
 ## );
 ##
+## In this context 'lazy_load' means that object will
+## not be loaded at all. Loading of object data
+## will be delayed until its properties will be demanded.
+##
+## find_id() with 'lazy_load' mostly useful when utilized
+## by ORM->stat().
+##
+##   lazy_load == undef,   
+##   ,    
+## prefer_lazy_load().
+##
+##   find_id()   'lazy_load'  
+##       
+##   ,     
+##  $class,     find_id().
+##        finish_loading()
+##     ,     
+##         $class.
+##
+## finish_loading()     update()   
+## ,    .
+##
 sub find_id
 {
     my $class = shift;
@@ -326,7 +333,7 @@ sub find_id
             my $error = ORM::Error->new;
             $self->finish_loading( error=>$error );
             $self = undef if( ref $self eq 'ORM::Broken' || $error->fatal );
-            $error->upto( $arg{error} );
+            $arg{error} && $arg{error}->add( error=>$error );
         }
 
         $self && $class->_cache->add( $self );
@@ -389,7 +396,7 @@ sub find_or_new
         }
     }
 
-    $error->upto( $arg{error} );
+    $arg{error} && $arg{error}->add( error=>$error );
     return $error->fatal ? undef : $obj[0];
 }
 
@@ -400,15 +407,26 @@ sub find_or_new
 ## use: $ta = $class->new_transaction( error=>ORM::Error );
 ##
 ## Begins transaction.
-## Transaction commits when object $ta is destroyed.
+## Transaction will commit when object $ta will be destroyed.
 ##
 sub new_transaction
 {
-    my $class  = shift;
-    my $iclass = $class->initial_class;
-    my %arg    = @_;
+    my $self = shift;
+    my %arg  = @_;
+    my $my_ta;
 
-    ORM::Ta->new( class=>$iclass, error=>$arg{error} );
+    if( $ta )
+    {
+        $my_ta = ORM::Ta->new;
+    }
+    else
+    {
+        $my_ta = ORM::Ta->new( db=>$db, error=>$arg{error} );
+        $ta    = $my_ta;
+        weaken $ta;
+    }
+
+    return $my_ta;
 }
 
 ## use: $self->update
@@ -425,7 +443,6 @@ sub update
     my $class    = ref $self;
     my %arg      = @_;
     my $error    = ORM::Error->new;
-    my $ta       = $class->new_transaction( error=>$error );
     my $history  = defined $arg{history} ? $arg{history} : $class->history_is_enabled;
     my %changed_prop;
     my %expr_prop;
@@ -493,9 +510,9 @@ sub update
         }
     }
     # User validations
-    if( %changed_prop && !$error->fatal )
+    unless( $error->fatal )
     {
-        $self->_validate_prop( prop=>\%changed_prop, old=>\%old_prop, method=>'update', error=>$error );
+        $self->_validate_prop( prop=>\%changed_prop, error=>$error );
     }
     # Detect data changes again to consider changes in _validate_prop
     unless( $error->fatal )
@@ -515,7 +532,7 @@ sub update
         }
     }
 
-    if( !$self->is_temporary && !$error->fatal && scalar( %changed_prop ) )
+    if( $self->id && !$error->fatal && scalar( %changed_prop ) )
     {
         for my $prop ( keys %expr_prop )
         {
@@ -525,7 +542,7 @@ sub update
         # Update object
         unless( $error->fatal )
         {
-            $class->_db->update_object
+            $db->update_object
             (
                 object     => $self,
                 values     => \%changed_prop,
@@ -550,7 +567,7 @@ sub update
                     $self->{_ORM_data}{$prop_name}
                 ];
             }
-            $class->_history_class->new
+            $history_class->new
             (
                 error   => $error,
                 obj     => $self,
@@ -565,7 +582,7 @@ sub update
         $self->{_ORM_data} = \%old_prop;
     }
 
-    $error->upto( $arg{error} );
+    $arg{error} && $arg{error}->add( error=>$error );
     return undef;
 }
 
@@ -577,26 +594,25 @@ sub delete
     my $class   = ref $self;
     my %arg     = @_;
     my $error   = ORM::Error->new;
-    my $ta      = $class->new_transaction( error=>$error );
     my $history = defined $arg{history} ? $arg{history} : $class->history_is_enabled;
 
-    unless( $self->is_temporary )
+    if( $self->id )
     {
         unless( $error->fatal )
         {
             # Make record in history
             if( $history )
             {
-                $class->_history_class->new( obj=>$self, deleted=>1, error=>$error );
+                $history_class->new( obj=>$self, deleted=>1, error=>$error );
             }
         }
         unless( $error->fatal )
         {
-            $class->_db->delete_object
+            $db->delete_object
             (
                 object => $self,
                 error  => $error,
-                emulate_foreign_keys => $class->_emulate_foreign_keys,
+                emulate_foreign_keys => $emulate_foreign_keys,
             );
         }
         unless( $error->fatal )
@@ -605,7 +621,7 @@ sub delete
         }
     }
 
-    $error->upto( $arg{error} );
+    $arg{error} && $arg{error}->add( error=>$error );
     return undef;
 }
 
@@ -665,9 +681,9 @@ sub finish_loading
     )
     {
         my $error = ORM::Error->new;
-        my $data  = $class->_db->select_tables
+        my $data  = $db->select_tables
         (
-            id     => $self->qc( $self->id ),
+            id     => $self->id,
             tables => $self->{_ORM_missing_tables},
             error  => $error,
         );
@@ -711,25 +727,17 @@ sub finish_loading
     if( $new_class && $new_class ne $class )
     {
         $class->_load_ORM_class( $new_class );
+        bless $self, $new_class;
 
-        if( UNIVERSAL::isa( $new_class, $class ) )
+        my $base_class_tables = $class->_db_tables_count;
+        my $class_tables      = $new_class->_db_tables_count;
+
+        for( my $i=$base_class_tables; $i<$class_tables; $i++ )
         {
-            bless $self, $new_class;
-
-            my $base_class_tables = $class->_db_tables_count;
-            my $class_tables      = $new_class->_db_tables_count;
-
-            for( my $i=$base_class_tables; $i<$class_tables; $i++ )
-            {
-                $self->{_ORM_missing_tables}{$new_class->_db_table($i)} = 1;
-            }
-
-            $self->finish_loading( error=>$arg{error} ) unless( defined $prop );
-        }
-        else
-        {
-            $self->_rebless_to_broken( deleted=>1 );
+            $self->{_ORM_missing_tables}{$new_class->_db_table($i)} = 1;
         }
+
+        $self->finish_loading( error=>$arg{error} ) unless( defined $prop );
     }
 }
 
@@ -739,7 +747,7 @@ sub finish_loading
 
 sub id           { $_[0]->{_ORM_data}{id}; }
 sub class        { ref $_[0] || $_[0]; }
-sub is_temporary { $_[0]->{_ORM_tpm}; }
+sub is_temporary { ! $_[0]->{_ORM_data}{id}; }
 
 sub __ORM_db_value { $_[0]->{_ORM_data}{id}; }
 sub __ORM_new_db_value
@@ -756,15 +764,23 @@ sub __ORM_new_db_value
     return $self;
 }
 
-sub _class_info
+sub base_class
 {
-    my $class = ref $_[0] || $_[0];
-    $class->_class_hier->{$class};
+    my $class = ref $_[0] ? ref shift : shift;
+    $hier{$class}{BASE_CLASS}
 }
 
-sub base_class    { $_[0]->_class_info->{BASE_CLASS}; }
-sub primary_class { $_[0]->_class_info->{PRIMARY_CLASS}; }
-sub initial_class { $_[0]->_is_initial ? $_[0] : $_[0]->_class_info->{INITIAL_CLASS}; }
+sub primary_class
+{
+    my $class = ref $_[0] ? ref shift : shift;
+    $hier{$class}{PRIMARY_CLASS}
+}
+
+sub initial_class
+{
+    my $class = ref $_[0] ? ref shift : shift;
+    $hier{$class}{INITIAL_CLASS}
+}
 
 sub M
 {
@@ -797,13 +813,21 @@ sub P
     );
 }
 
-sub metaprop_class { $_[0]->_class_info->{METAPROP_CLASS}; }
+sub metaprop_class
+{
+    my $self  = shift;
+    my $class = ( ref $self ) || $self;
 
-sub ql { $_[0]->_db->ql( $_[1] ); }
-sub qc { $_[0]->_db->qc( $_[1] ); }
-sub qi { $_[0]->_db->qi( $_[1] ); }
-sub qt { $_[0]->_db->qt( $_[1] ); }
-sub qf { $_[0]->_db->qf( $_[1] ); }
+    return $hier{$class}{METAPROP_CLASS};
+}
+
+sub ql { $db->ql( $_[1] ); }
+sub qc { $db->qc( $_[1] ); }
+sub qi { $db->qi( $_[1] ); }
+sub qt { $db->qt( $_[1] ); }
+sub qf { $db->qf( $_[1] ); }
+sub _db { $db; }
+sub _history_class { $history_class; }
 
 ## use: $state = $class->history_is_enabled;
 ## use: $state = $class->history_is_enabled( $new_state );
@@ -821,9 +845,9 @@ sub history_is_enabled
     {
         if( defined $_[0] )
         {
-            if( $class->_class_info )
+            if( $hier{$class} )
             {
-                $class->_class_info->{HISTORY_IS_ENABLED} = $_[0];
+                $hier{$class}{HISTORY_IS_ENABLED} = $_[0];
             }
             else
             {
@@ -832,13 +856,13 @@ sub history_is_enabled
         }
         else
         {
-            delete $class->_class_info->{HISTORY_IS_ENABLED} if( $class->_class_info );
+            delete $hier{$class}{HISTORY_IS_ENABLED} if( $hier{$class} );
         }
     }
 
-    exists $class->_class_info->{HISTORY_IS_ENABLED}
-        ? $class->_class_info->{HISTORY_IS_ENABLED}
-        : $class->_history_class;
+    exists $hier{$class}{HISTORY_IS_ENABLED}
+        ? $hier{$class}{HISTORY_IS_ENABLED}
+        : $history_class;
 }
 
 ## use: $state = $class->prefer_lazy_load;
@@ -857,17 +881,17 @@ sub prefer_lazy_load
     {
         if( defined $_[0] )
         {
-            $class->_class_info->{PREFER_LAZY_LOAD} = $_[0];
+            $hier{$class}{PREFER_LAZY_LOAD} = $_[0];
         }
         else
         {
-            delete $class->_class_info->{PREFER_LAZY_LOAD};
+            delete $hier{$class}{PREFER_LAZY_LOAD};
         }
     }
 
-    exists $class->_class_info->{PREFER_LAZY_LOAD}
-        ? $class->_class_info->{PREFER_LAZY_LOAD}
-        : $class->_default_prefer_lazy_load;
+    exists $hier{$class}{PREFER_LAZY_LOAD}
+        ? $hier{$class}{PREFER_LAZY_LOAD}
+        : $prefer_lazy_load;
 }
 
 sub _plain_prop
@@ -875,8 +899,8 @@ sub _plain_prop
     my $class = shift;
     my $prop  = shift;
 
-    exists( $class->_class_info->{PROP}{$prop} )
-        && ( ! $class->_class_info->{PROP}{$prop} );
+    exists( $hier{$class}{PROP}{$prop} )
+        && ( ! $hier{$class}{PROP}{$prop} );
 }
 sub _prop_is_ref
 {
@@ -884,36 +908,36 @@ sub _prop_is_ref
     my $prop   = shift;
     my $pclass = $class->_prop_class( $prop );
 
-    $pclass && $class->_class_hier->{$pclass} && $pclass;
+    $pclass && $hier{$pclass} && $pclass;
 }
 
-sub _is_sealed                { $_[0]->_class_info->{SEALED}; }
-sub _prop_class               { $_[0]->_class_info->{PROP}{$_[1]}; }
-sub _prop_default_value       { $_[0]->_class_info->{PROP_DEFAULT_VALUE}{$_[1]}; }
-sub _has_prop                 { exists $_[0]->_class_info->{PROP}{$_[1]}; }
-sub _prop2table               { $_[0]->_class_info->{PROP2TABLE_MAP}{$_[1]}; }
-sub _prop2field               { $_[0]->_class_info->{PROP2FIELD_MAP}{$_[1]}; }
-sub _is_intermediate          { $_[0]->_class_info->{INTERMEDIATE}; }
-sub _is_initial               { !$_[0]->_class_info; }
-sub _db_table                 { $_[0]->_class_info->{TABLE}[$_[1]]; }
-sub _db_tables_str            { $_[0]->_class_info->{TABLES_STR}; }
-sub _db_tables_count          { scalar( @{$_[0]->_class_info->{TABLE}} ); }
-sub _db_tables                { @{$_[0]->_class_info->{TABLE}}; }
-sub _db_tables_ref            { $_[0]->_class_info->{TABLE}; }
-sub _db_table_fields          { keys %{$_[0]->_class_info->{TABLE_STRUCT}{$_[1]}}; }
-sub _db_tables_inner_join     { $_[0]->_class_info->{TABLES_INNER_JOIN}; }
-sub _not_mandatory_props      { keys %{$_[0]->_class_info->{PROP2FIELD_MAP}}; }
-sub _all_props                { ( 'id', 'class', keys %{$_[0]->_class_info->{PROP2FIELD_MAP}} ); }
-sub _cache                    { $_[0]->primary_class->_class_info->{CACHE}; }
+sub _is_sealed                { $hier{ref $_[0]||$_[0]}{SEALED}; }
+sub _prop_class               { $hier{$_[0]}{PROP}{$_[1]}; }
+sub _prop_default_value       { $hier{$_[0]}{PROP_DEFAULT_VALUE}{$_[1]}; }
+sub _has_prop                 { exists $hier{$_[0]}{PROP}{$_[1]}; }
+sub _prop2table               { $hier{$_[0]}{PROP2TABLE_MAP}{$_[1]}; }
+sub _prop2field               { $hier{$_[0]}{PROP2FIELD_MAP}{$_[1]}; }
+sub _is_intermediate          { $hier{$_[0]}{INTERMEDIATE}; }
+sub _is_initial               { !$hier{$_[0]}; }
+sub _db_table                 { $hier{$_[0]}{TABLE}[$_[1]]; }
+sub _db_tables_str            { $hier{$_[0]}{TABLES_STR}; }
+sub _db_tables_count          { scalar( @{$hier{$_[0]}{TABLE}} ); }
+sub _db_tables                { @{$hier{$_[0]}{TABLE}}; }
+sub _db_tables_ref            { $hier{$_[0]}{TABLE}; }
+sub _db_table_fields          { keys %{$hier{$_[0]}{TABLE_STRUCT}{$_[1]}}; }
+sub _db_tables_inner_join     { $hier{$_[0]}{TABLES_INNER_JOIN}; }
+sub _not_mandatory_props      { keys %{$hier{$_[0]}{PROP2FIELD_MAP}}; }
+sub _all_props                { ( 'id', 'class', keys %{$hier{$_[0]}{PROP2FIELD_MAP}} ); }
+sub _cache                    { $hier{( ref $_[0] || $_[0] )->primary_class}{CACHE}; }
 
 sub _rev_refs
 {
     my $class = shift;
-    my @refs  = values %{$class->_class_info->{REV_REFS}};
+    my @refs  = values %{$hier{$class}{REV_REFS}};
 
-    if( $class->_class_info->{BASE_CLASS} )
+    if( $hier{$class}{BASE_CLASS} )
     {
-        push @refs, $class->_class_info->{BASE_CLASS}->_rev_refs;
+        push @refs, $hier{$class}{BASE_CLASS}->_rev_refs;
     }
 
     return @refs;
@@ -925,7 +949,7 @@ sub _has_rev_ref
     my $rev_class = shift;
     my $rev_prop  = shift;
 
-    $class->_class_info->{REV_REFS}{ $rev_class.' '.$rev_prop }
+    $hier{$class}{REV_REFS}{ $rev_class.' '.$rev_prop }
     || (
         $rev_class->base_class
         && $class->_has_rev_ref( $rev_class->base_class, $rev_prop )
@@ -952,6 +976,11 @@ sub _has_rev_ref
 ##     return_res  => boolean,
 ## )
 ##
+##  'count' ,   
+##        
+##  . page  pagesize   
+##   count.
+##
 sub stat
 {
     my $class    = shift;
@@ -1031,7 +1060,7 @@ sub stat
         }
 
         # Fetch result set
-        $res = $class->_db->select_stat
+        $res = $db->select_stat
         (
             class       => $class,
             data        => \%data,
@@ -1045,7 +1074,6 @@ sub stat
         );
     }
 
-    # Final step, prepare resulting data
     if( $res && !$error->fatal )
     {
         if( $arg{count} )
@@ -1082,14 +1110,14 @@ sub stat
         }
     }
 
-    $error->upto( $arg{error} );
+    $arg{error} && $arg{error}->add( error=>$error );
     return $res;
 }
 
 ## use: $prop = $obj->_property
 ## (
-##     name      => string,
-##     error     => ORM::Error,
+##     name     => string,
+##     error    => ORM::Error,
 ## );
 ##
 ## 'name'   - is name of the property corresponding to field name in DB table
@@ -1098,7 +1126,6 @@ sub stat
 ##            either object referenced by id in DB,
 ##            or object referenced by value in DB
 ##
-sub _prop { shift->_property( @_ ); }
 sub _property
 {
     my $self   = shift;
@@ -1149,7 +1176,7 @@ sub _property
         }
     }
 
-    $error->upto( $arg{error} );
+    $arg{error} && $arg{error}->add( error=>$error );
     return $res;
 }
 
@@ -1165,13 +1192,11 @@ sub _property
 ##            either object referenced by id in DB,
 ##            or object referenced by value in DB
 ##
-sub _prop_id { shift->_property_id( @_ ); }
 sub _property_id
 {
     my $self   = shift;
     my %arg;
     my $prop;
-    my $value;
 
     if( @_ == 1 )
     {
@@ -1183,50 +1208,12 @@ sub _property_id
         $prop = $arg{name};
     }
 
-    if( $prop eq 'class' )
+    if( exists $self->{_ORM_missing_tables} )
     {
-        $value = $self->class;
-    }
-    else
-    {
-        if( exists $self->{_ORM_missing_tables} )
-        {
-            $self->finish_loading( prop=>$prop, error=>$arg{error} );
-        }
-        $value = $self->{_ORM_data}{$prop};
+        $self->finish_loading( prop=>$prop, error=>$arg{error} )
     }
 
-    return $value;
-}
-
-sub _rev { shift->_rev_prop( @_ ); }
-sub _rev_prop
-{
-    my $self      = shift;
-    my $rev_class = shift;
-    my $rev_prop  = shift;
-    my %arg       = @_;
-
-    if( (ref $self)->_has_rev_ref( $rev_class, $rev_prop ) )
-    {
-        $arg{filter} = $arg{filter} & ( $rev_class->M->_prop( $rev_prop ) == $self );
-        $rev_class->find( %arg );
-    }
-}
-
-sub _rev_count { shift->_rev_prop_count( @_ ); }
-sub _rev_prop_count
-{
-    my $self      = shift;
-    my $rev_class = shift;
-    my $rev_prop  = shift;
-    my %arg       = @_;
-
-    if( (ref $self)->_has_rev_ref( $rev_class, $rev_prop ) )
-    {
-        $arg{filter} = $arg{filter} & ( $rev_class->M->_prop( $rev_prop ) == $self );
-        $rev_class->count( %arg );
-    }
+    return $self->{_ORM_data}{$prop};
 }
 
 ## use: $prop = $obj->prop( error=>ORM::Error, new_value=>SCALAR );
@@ -1244,7 +1231,7 @@ sub AUTOLOAD
         my $self = shift;
         my %arg  = @_;
 
-        croak "Called undefined static method '$ORM::AUTOLOAD' of class '$self'" unless( ref $self );
+        die "Called undefined static method '$ORM::AUTOLOAD' of class '$self'" unless( ref $self );
 
         $self->_property( name=>$prop, %arg );
     }
@@ -1257,7 +1244,7 @@ sub AUTOLOAD
 sub optimize_storage
 {
     my $class = shift;
-    $class->_db->optimize_tables( class=>$class );
+    $db->optimize_tables( class=>$class );
 }
 
 ##
@@ -1275,7 +1262,7 @@ sub _find_constructor
     {
         if( $prop->{class} )
         {
-            $class->_load_ORM_class( $prop->{class} );
+            ORM->_load_ORM_class( $prop->{class} );
             $self = bless { _ORM_data => $prop }, $prop->{class};
 
             if( $result_tables )
@@ -1318,7 +1305,6 @@ sub _rebless_to_broken
         $self->{error} = $arg{error};
     }
 
-    delete $self->{_ORM_tmp};
     delete $self->{_ORM_data};
     delete $self->{_ORM_cache};
     delete $self->{_ORM_missing_tables};
@@ -1328,8 +1314,10 @@ sub _rebless_to_broken
 
 ## use: $self->_normalize_prop_to_db_value( name=>STRING, value=>SCALAR, error=>ORM::Error )
 ##
-## Normalize specified value to be able to store it in database table.
-## All arguments are necessary.
+##    ()  
+##        .
+##
+##   .
 ##
 sub _normalize_prop_to_db_value
 {
@@ -1405,7 +1393,7 @@ sub _normalize_prop_to_db_value
                 value => $prop_value,
                 error => $error,
             );
-            $prop_value = defined $obj ? $obj->__ORM_db_value : undef;
+            $prop_value = $obj ? $obj->__ORM_db_value : undef;
         }
         elsif( UNIVERSAL::isa( $prop_ref, $class->_prop_class( $prop_name ) ) )
         {
@@ -1426,7 +1414,20 @@ sub _normalize_prop_to_db_value
     return $arg{error}->fatal ? undef : $prop_value;
 }
 
-## use: $self->_validate_prop( prop=>HASH, method=>string, error=>ORM::Error )
+## use: $self->_validate_prop( prop=>HASH, error=>ORM::Error )
+##
+##     ,
+##       .
+##
+##  'error' - .
+##  'prop'     ,  
+##    ,  .
+##
+##      new  update,
+##        .
+##    _validate_prop  
+##       
+##  $self->SUPER::_validate_prop(...).
 ##
 sub _validate_prop {}
 
@@ -1455,7 +1456,7 @@ sub _fix_prop
         }
     }
 
-    $error->upto( $arg{error} );
+    $arg{error} && $arg{error}->add( error=>$error );
     return undef;
 }
 
@@ -1465,28 +1466,37 @@ sub _fix_prop
 ##     history_class        => string||undef,
 ##     prefer_lazy_load     => boolean,
 ##     emulate_foreign_keys => boolean,
-##     default_cache_size   => integer,
 ## )
 ##
+## _init      
+##       .
+##
 sub _init
 {
     my $class = shift;
     my %arg   = @_;
 
-    die "'db' argument not specified"                   unless( exists $arg{db} );
-    die "'db' argument is specified but undefined"      unless( $arg{db} );
-    die "'db' argument specified is not descendant of 'ORM::Db'" unless( UNIVERSAL::isa( $arg{db}, 'ORM::Db' ) );
-    die "'prefer_lazy_load' argument not specified"     unless( exists $arg{prefer_lazy_load} );
-    die "'emulate_foreign_keys' argument not specified" unless( exists $arg{emulate_foreign_keys} );
-    die "'default_cache_size' argument not specified"   unless( exists $arg{default_cache_size} );
+    if( $initialized )
+    {
+        die "At present it is imposible to derive more that one control class from ORM";
+    }
+    else
+    {
+        die "'db' argument not specified"                   unless( exists $arg{db} );
+        die "'db' argument is specified but undefined"      unless( $arg{db} );
+        die "'db' argument specified is not descendant of 'ORM::Db'" unless( UNIVERSAL::isa( $arg{db}, 'ORM::Db' ) );
+        die "'prefer_lazy_load' argument not specified"     unless( exists $arg{prefer_lazy_load} );
+        die "'emulate_foreign_keys' argument not specified" unless( exists $arg{emulate_foreign_keys} );
+        die "'default_cache_size' argument not specified" unless( exists $arg{default_cache_size} );
 
-    $class->_class_hier( {} );
-    $class->_db( $arg{db} );
-    $class->_history_class( $arg{history_class} );
-    $class->_default_prefer_lazy_load( $arg{prefer_lazy_load} );
-    $class->_emulate_foreign_keys( $arg{emulate_foreign_keys} );
-    $class->_default_cache_size( $arg{default_cache_size} );
-    $class->_current_transaction( undef );
+        $initialized = 1;
+
+        $db                   = $arg{db};
+        $history_class        = $arg{history_class};
+        $prefer_lazy_load     = $arg{prefer_lazy_load};
+        $emulate_foreign_keys = $arg{emulate_foreign_keys};
+        $default_cache_size   = $arg{default_cache_size};
+    }
 }
 
 ## use: $base_class->_derive
@@ -1499,19 +1509,20 @@ sub _init
 ##     prefer_lazy_load   => boolean,
 ## )
 ##
+## Should be called in BEGIN{...} of every derived class
+##
 sub _derive
 {
     my $class   = shift;
     my %arg     = @_;
     my $error   = ORM::Error->new;
-    my $base    = $class->_class_info;
+    my $base    = $hier{$class};
     my $derived;
     my $struct;
     my $defaults;
     my $table;
 
-    $derived = {};
-    $class->_class_hier->{$arg{derived_class}} = $derived;
+    $derived = $hier{$arg{derived_class}} = {};
 
     # Copy SQL configuration from base class
     if( $base )
@@ -1546,7 +1557,7 @@ sub _derive
     {
         $derived->{INITIAL_CLASS} = $class;
         $derived->{PRIMARY_CLASS} = $arg{derived_class};
-        $derived->{CACHE}         = ORM::Cache->new( size=>($arg{cache_size}||$class->_default_cache_size) );
+        $derived->{CACHE}         = ORM::Cache->new( size=>($arg{cache_size}||$default_cache_size) );
     }
 
     unless( $error->fatal )
@@ -1569,22 +1580,22 @@ sub _derive
         {
             $derived->{PREFER_LAZY_LOAD} = $arg{prefer_lazy_load};
         }
-
+            
         # Detect db table name
         $table = $arg{table} || $class->_guess_table_name( $arg{derived_class} );
     }
 
     if( $table )
     {
-        ( $struct, $defaults ) = $class->_db->table_struct
+        ( $struct, $defaults ) = $db->table_struct
         (
             class => $arg{derived_class},
             table => $table,
             error => $error,
         );
-        if( $class->_history_class && $arg{derived_class} eq $class->_history_class )
+        if( $arg{derived_class} eq $history_class )
         {
-            $struct->{slaved_by} = $class->_history_class;
+            $struct->{slaved_by} = $history_class;
         }
         # Check whether table exists
         if( ! scalar( %$struct ) )
@@ -1624,7 +1635,7 @@ sub _derive
             {
                 $derived->{TABLES_INNER_JOIN} .= ' AND ' if( $derived->{TABLES_INNER_JOIN} );
                 $derived->{TABLES_INNER_JOIN} .=
-                    $class->_db->qt( $table ).'.id = '.$class->_db->qt( $derived->{TABLE}[0] ).'.id';
+                    $db->qt( $table ).'.id = '.$db->qt( $derived->{TABLE}[0] ).'.id';
             }
         }
         # Initialize
@@ -1652,7 +1663,7 @@ sub _derive
                     if( $field ne 'class' )
                     {
                         $derived->{PROP2FIELD_MAP}{$field} =
-                            $class->_db->qt( $table ) . '.' . $class->_db->qf( $field );
+                            $db->qt( $table ) . '.' . $db->qf( $field );
                     }
                 }
                 else
@@ -1661,7 +1672,7 @@ sub _derive
                     (
                         "Duplicate columns "
                         . "'$derived->{PROP2FIELD_MAP}{$field}',"
-                        . " '".$class->_db->qt($table).'.'.$class->_db->qf($field)."'"
+                        . " '".$db->qt($table).'.'.$db->qf($field)."'"
                     );
                     last;
                 }
@@ -1678,7 +1689,7 @@ sub _derive
             {
                 $derived->{TABLES_STR} .= ',';
             }
-            $derived->{TABLES_STR} .= $class->_db->qt( $table );
+            $derived->{TABLES_STR} .= $db->qt( $table );
             $derived->{TABLE_STRUCT}{$table} = $struct;
             push @{$derived->{TABLE}}, $table;
         }
@@ -1710,32 +1721,32 @@ sub _derive
         for my $prop ( keys %{$derived->{TABLE_STRUCT}{$table}} )
         {
             my $pclass = $derived->{PROP}{$prop};
-            if( $pclass && !$class->_class_hier->{$pclass} )
+            if( $pclass && !$hier{$pclass} )
             {
                 $require{$pclass} = 1;
             }
         }
-        for my $pclass ( $class->_db->referencing_classes( class=>$arg{derived_class}, error=>$error ) )
+        for my $pclass ( $db->referencing_classes( class=>$arg{derived_class}, error=>$error ) )
         {
-            $require{$pclass->{class}} = 1 unless( $class->_class_hier->{$pclass->{class}} );
+            $require{$pclass->{class}} = 1 unless( $hier{$pclass->{class}} );
             $derived->{REV_REFS}{ $pclass->{class}.' '.$pclass->{prop} }
                 = [ $pclass->{class}, $pclass->{prop} ];
         }
-        ## Following pease of code make sence only in mod_perl environment,
-        ## it is necessary to avoid the following problem:
+        ##          mod_perl,
+        ##       .
         ##
-        ## If you have created and loaded new ORM-class My::Class2 that contain
-        ## referencing property to class My::Class1, then My::Class1 does not
-        ## know about new referer and therefore My::Class1->_rev_refs returns
-        ## outdated data.
+        ##     ORM-   ,
+        ##           ,
+        ## ..     _rev_refs   
+        ## httpd.     .
         ##
         for my $prop ( keys %{$derived->{TABLE_STRUCT}{$table}} )
         {
             my $pclass = $derived->{PROP}{$prop};
             my $key    = "$arg{derived_class} $prop";
-            if( $pclass && $class->_class_hier->{$pclass} && !$pclass->_class_info->{REV_REFS}{$key} )
+            if( $pclass && $hier{$pclass} && !$hier{$pclass}{REV_REFS}{$key} )
             {
-                $pclass->_class_info->{REV_REFS}{$key} = [ $arg{derived_class}, $prop ];
+                $hier{$pclass}->{REV_REFS}{$key} = [ $arg{derived_class}, $prop ];
             }
         }
 
@@ -1743,7 +1754,7 @@ sub _derive
         for my $prop ( keys %{$derived->{TABLE_STRUCT}{$table}} )
         {
             my $pclass = $derived->{PROP}{$prop};
-            if( $pclass && !$class->_class_hier->{$pclass} )
+            if( $pclass && !$hier{$pclass} )
             {
                 ORM::Metaprop->_class2metaclass( $pclass );
             }
@@ -1771,11 +1782,11 @@ sub _values_are_not_equal
 }
 
 ##
-## METHODS AND PROPERTIES TO USE DURING CLASS INITIALISATION
-## ( ORM->_derive )
+## METHODS AND PROPERTIES TO USE VIA CLASS INITIALISATION
+## (ORM->_derive)
 ##
 
-sub _class_is_primary         { ! exists $_[1]->_class_info->{TABLE}; }
+sub _class_is_primary         { ! exists $hier{ $_[1] }{TABLE}; }
 
 ## use: $table_name = $class->_guess_table_name( $obj_class );
 ##
@@ -1784,7 +1795,7 @@ sub _guess_table_name
     my $class = shift;
     my $table = shift;
 
-    $table =~ s/::/_/g;
+    $table =~ s/::/__/g;
 
     return $table;
 }
@@ -1818,10 +1829,6 @@ sub _db_type_to_class
     {
         $prop_class = 'ORM::Datetime';
     }
-    elsif( ( lc $type ) eq 'timestamp' )
-    {
-        $prop_class = 'ORM::Datetime';
-    }
 
     return $prop_class;
 }
@@ -1833,7 +1840,7 @@ sub _load_ORM_class
     my $class      = shift;
     my $load_class = shift;
 
-    unless( $class->_class_hier->{$load_class} )
+    unless( $hier{$load_class} )
     {
         $load_class .= '.pm';
         $load_class  =~ s(::)(/)g;
@@ -1843,7 +1850,7 @@ sub _load_ORM_class
 
 sub DESTROY
 {
-    exists $_[0]->_class_hier->{PRIMARY_CLASS} && $_[0]->_cache && $_[0]->_cache->delete( $_[0] );
+    $_[0]->_cache && $_[0]->_cache->delete( $_[0] );
 }
 
 1;
