@@ -1,37 +1,21 @@
 #
-# DESCRIPTION
+# DESCRIPTION:
 #   PerlORM - Object relational mapper (ORM) for Perl. PerlORM is Perl
 #   library that implements object-relational mapping. Its features are
 #   much similar to those of Java's Hibernate library, but interface is
 #   much different and easier to use.
 #
-# AUTHOR
+# AUTHOR:
 #   Alexey V. Akimov <akimov_alexey@sourceforge.net>
 #
 # COPYRIGHT
-#   Copyright (C) 2005-2006 Alexey V. Akimov
-#
-#   This library is free software; you can redistribute it and/or
-#   modify it under the terms of the GNU Lesser General Public
-#   License as published by the Free Software Foundation; either
-#   version 2.1 of the License, or (at your option) any later version.
-#   
-#   This library is distributed in the hope that it will be useful,
-#   but WITHOUT ANY WARRANTY; without even the implied warranty of
-#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#   Lesser General Public License for more details.
-#   
-#   You should have received a copy of the GNU Lesser General Public
-#   License along with this library; if not, write to the Free Software
-#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+#   Copyright (c) 2005 Alexey V. Akimov. All rights reserved.
 #
 
 package ORM::Base;
 
 use Carp;
 
-$VERSION = 0.8;
-
 my %require;
 my %loaded;
 my $active = 0;
@@ -68,10 +52,7 @@ sub import
     print STDERR "  Loading class $derived\n" if( $debug );
 
     my @require = $base->_derive( derived_class=>$derived, %arg );
-    if( $derived->_history_class && !$loaded{$derived->_history_class} )
-    {
-        push @require, $derived->_history_class;
-    }
+    push @require, $derived->_history_class unless( $loaded{$derived->_history_class} );
     for my $module ( @require )
     {
         if( $loaded{$module} )
