@@ -1,34 +1,20 @@
 #
-# DESCRIPTION
+# DESCRIPTION:
 #   PerlORM - Object relational mapper (ORM) for Perl. PerlORM is Perl
 #   library that implements object-relational mapping. Its features are
 #   much similar to those of Java's Hibernate library, but interface is
 #   much different and easier to use.
 #
-# AUTHOR
+# AUTHOR:
 #   Alexey V. Akimov <akimov_alexey@sourceforge.net>
 #
 # COPYRIGHT
-#   Copyright (C) 2005-2006 Alexey V. Akimov
-#
-#   This library is free software; you can redistribute it and/or
-#   modify it under the terms of the GNU Lesser General Public
-#   License as published by the Free Software Foundation; either
-#   version 2.1 of the License, or (at your option) any later version.
-#   
-#   This library is distributed in the hope that it will be useful,
-#   but WITHOUT ANY WARRANTY; without even the implied warranty of
-#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#   Lesser General Public License for more details.
-#   
-#   You should have received a copy of the GNU Lesser General Public
-#   License along with this library; if not, write to the Free Software
-#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+#   Copyright (c) 2005 Alexey V. Akimov. All rights reserved.
 #
 
 package ORM::Db::DBI::MySQL;
 
-$VERSION = 0.83;
+$VERSION = 0.8;
 
 use base 'ORM::Db::DBI';
 
@@ -40,14 +26,9 @@ sub new
 {
     my $class = shift;
     my %arg   = @_;
-    my $self;
 
     $arg{driver} = 'mysql';
-    $self = $class->SUPER::new( %arg );
-
-    $self->{allow_nontransactional_tables} = $arg{allow_nontransactional_tables};
-
-    return $self;
+    $class->SUPER::new( %arg );
 }
 
 ##
@@ -62,9 +43,6 @@ sub qc
     if( defined $str )
     {
         $str =~ s/\'/\'\'/g;
-        $str =~ s/\\/\\\\/g;
-        $str =~ s/\n/\\n/g;
-        $str =~ s/\r/\\r/g;
         $str = "'$str'";
     }
     else
@@ -93,36 +71,6 @@ sub qf { $_[0]->qi( $_[1] ); }
 ## OBJECT METHODS
 ##
 
-sub begin_transaction
-{
-    my $self  = shift;
-    my %arg   = @_;
-
-    $self->{ta} = 1;
-    $self->do( query=>"START TRANSACTION", error=>$arg{error} );
-}
-
-sub commit_transaction
-{
-    my $self  = shift;
-    my %arg   = @_;
-
-    delete $self->{ta};
-    $self->do( query=>"COMMIT", error=>$arg{error} );
-}
-
-sub rollback_transaction
-{
-    my $self  = shift;
-    my %arg   = @_;
-
-    delete $self->{ta};
-    unless( $self->{lost_connection} )
-    {
-        $self->do( query=>"ROLLBACK", error=>$arg{error} );
-    }
-}
-
 sub update_object
 {
     my $self = shift;
@@ -130,26 +78,37 @@ sub update_object
     $self->update_object_part( all_tables=>1, @_ );
 }
 
-## Can be optimized to check consistency
-## right in DELETE statement using JOINS
-##
 sub delete_object
 {
-    my $self      = shift;
-    my %arg       = @_;
+    my $self = shift;
+    my %arg  = @_;
     my $obj       = $arg{object};
     my $obj_class = ref $obj;
-    my $error     = ORM::Error->new;
-    my $ta        = $obj_class->new_transaction( error=>$error );
     my $join      = $obj_class->_db_tables_inner_join;
+    my $error     = ORM::Error->new;
     my $rows_affected;
 
-    $self->check_object_referers
-    (
-        object => $obj,
-        error  => $error,
-        check  => $arg{emulate_foreign_keys},
-    );
+    # Should be optimized!
+    if( $arg{emulate_foreign_keys} )
+    {
+        for my $ref ( $obj_class->_rev_refs )
+        {
+            my $referers = $ref->[0]->count
+            (
+                filter => ( $ref->[0]->M->_prop($ref->[1])==$obj->id ),
+                error  => $error,
+            );
+            if( $referers )
+            {
+                $error->add_fatal
+                (
+                    "Can't delete instance ID#" . $obj->id
+                    . " of '$obj_class', because there're "
+                    . "$referers instances of '$ref->[0]' refer to it."
+                );
+            }
+        }
+    }
 
     unless( $error->fatal )
     {
@@ -175,19 +134,9 @@ sub delete_object
                 . "', $rows_affected rows affected"
             );
         }
-
-        unless( $error->fatal )
-        {
-            $self->check_object_referers
-            (
-                object => $obj,
-                error  => $error,
-                check  => $arg{emulate_foreign_keys},
-            );
-        }
     }
 
-    $error->upto( $arg{error} );
+    $arg{error} && $arg{error}->add( error=>$error );
 }
 
 ## use: $id = $db->insertid()
@@ -219,7 +168,6 @@ sub table_struct
         }
     }
 
-    ## Check whether table's engine supports transactions
     unless( $error->fatal )
     {
         $res  = $self->select( error=>$error, query=>( 'SHOW TABLE STATUS LIKE '.$self->ql( $arg{table} ) ) );
@@ -231,7 +179,7 @@ sub table_struct
         {
             $error->add_fatal( 'Can\'t detect engine for "'.$arg{table}.'"' );
         }
-        elsif( !$self->{allow_nontransactional_tables} && $engine ne 'InnoDB' && $engine ne 'BDB' )
+        elsif( $engine ne 'InnoDB' && $engine ne 'BDB' )
         {
             $error->add_fatal
             (
@@ -246,7 +194,7 @@ sub table_struct
         $res = $self->select
         (
             error => $error,
-            query => 'SELECT * FROM '.$self->qt('_ORM_refs').' WHERE class='.$self->qc( $arg{class} ),
+            query => 'SELECT * FROM _ORM_refs WHERE class='.$self->qc( $arg{class} ),
         );
         unless( $error->fatal )
         {
@@ -260,7 +208,7 @@ sub table_struct
         }
     }
 
-    $error->upto( $arg{error} );
+    $arg{error} && $arg{error}->add( error=>$error );
     return \%field, \%defaults;
 }
 
@@ -292,9 +240,3 @@ sub pwd
         return undef;
     }
 }
-
-##
-## SQL FUNCTIONS
-##
-
-sub _func_concat        { shift; ORM::Filter::Func->new( 'CONCAT', @_ ); }
