@@ -1,29 +1,15 @@
 #
-# DESCRIPTION
+# DESCRIPTION:
 #   PerlORM - Object relational mapper (ORM) for Perl. PerlORM is Perl
 #   library that implements object-relational mapping. Its features are
 #   much similar to those of Java's Hibernate library, but interface is
 #   much different and easier to use.
 #
-# AUTHOR
+# AUTHOR:
 #   Alexey V. Akimov <akimov_alexey@sourceforge.net>
 #
 # COPYRIGHT
-#   Copyright (C) 2005-2006 Alexey V. Akimov
-#
-#   This library is free software; you can redistribute it and/or
-#   modify it under the terms of the GNU Lesser General Public
-#   License as published by the Free Software Foundation; either
-#   version 2.1 of the License, or (at your option) any later version.
-#   
-#   This library is distributed in the hope that it will be useful,
-#   but WITHOUT ANY WARRANTY; without even the implied warranty of
-#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#   Lesser General Public License for more details.
-#   
-#   You should have received a copy of the GNU Lesser General Public
-#   License along with this library; if not, write to the Free Software
-#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+#   Copyright (c) 2005 Alexey V. Akimov. All rights reserved.
 #
 
 use English;
@@ -150,7 +136,7 @@ sub new
         }
     }
 
-    $error->upto( $arg{error} );
+    $arg{error} && $arg{error}->add( error=>$error );
     return $record[0];
 }
 
@@ -158,19 +144,6 @@ sub new
 ## PROPERTIES
 ##
 
-sub obj
-{
-    my $self = shift;
-
-    unless( $self->{obj} )
-    {
-        $self->_load_ORM_class( $self->obj_class );
-        $self->{obj} = $self->obj_class->find_id( id=>$self->obj_id );
-    }
-
-    return $self->{obj};
-}
-
 sub master { ! $_[0]->slaved_by; }
 
 ##
@@ -313,7 +286,7 @@ sub rollback
             $self->delete( error=>$error );
         }
 
-        $error->upto( $arg{error} );
+        $arg{error} && $arg{error}->add( error=>$error );
     }
 }
 
