@@ -1,34 +1,20 @@
 #
-# DESCRIPTION
+# DESCRIPTION:
 #   PerlORM - Object relational mapper (ORM) for Perl. PerlORM is Perl
 #   library that implements object-relational mapping. Its features are
 #   much similar to those of Java's Hibernate library, but interface is
 #   much different and easier to use.
 #
-# AUTHOR
+# AUTHOR:
 #   Alexey V. Akimov <akimov_alexey@sourceforge.net>
 #
 # COPYRIGHT
-#   Copyright (C) 2005-2006 Alexey V. Akimov
-#
-#   This library is free software; you can redistribute it and/or
-#   modify it under the terms of the GNU Lesser General Public
-#   License as published by the Free Software Foundation; either
-#   version 2.1 of the License, or (at your option) any later version.
-#   
-#   This library is distributed in the hope that it will be useful,
-#   but WITHOUT ANY WARRANTY; without even the implied warranty of
-#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#   Lesser General Public License for more details.
-#   
-#   You should have received a copy of the GNU Lesser General Public
-#   License along with this library; if not, write to the Free Software
-#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+#   Copyright (c) 2005 Alexey V. Akimov. All rights reserved.
 #
 
 package ORM::Metaprop;
 
-$VERSION=0.81;
+$VERSION=0.8;
 
 use Carp;
 use ORM;
@@ -48,25 +34,14 @@ sub new
 {
     my $class = shift;
     my %arg   = @_;
-    my $self;
-
-    if( $class eq 'ORM::Metaprop' )
+    my $self  =
     {
-        $self = $arg{expr};
-    }
-    elsif( $arg{expr} )
-    {
-        $self =
-        {
-            expr       => $arg{expr},
-            tjoin      => $arg{expr}->_tjoin->copy,
-            prop_class => $class->_metaclass2class( $class ),
-        };
+        expr       => $arg{expr},
+        tjoin      => $arg{expr}->_tjoin->copy,
+        prop_class => $class->_metaclass2class( $class ),
+    };
 
-        bless $self, $class;
-    }
-
-    return $self;
+    return bless $self, $class;
 }
 
 ## use: $prop = $class->_new
@@ -76,19 +51,21 @@ sub new
 ## )
 ##
 ## prop_class:
-##  The class that 'prop' property belongs to.
+##  ,    .
 ##
 ## prop:
 ## 
-##  prop              =~ ( '->' DIRECT_PROPERTY | '-<' REVERSE_PROPERTY )+
-##  REVERSE_PROPERTY  =~ REFERRING_CLASS '.' CLASS_PROPERTY '.' ALIAS
+##  prop              =~ ( '->' _ | '-<' _ )+
+##  _ =~ _ '.' _ '.' 
 ## 
-##  DIRECT_PROPERTY   - property of the target class
-##  REVERSE_PROPERTY  - property of third class that refers to target class
-##  REFERRING_CLASS   - class that refers to target class by one of its properties
-##  CLASS_PROPERTY    - property name of the instance of the referring class
-##  ALIAS             - alpha-numeric string, aliasing allows to use different 
-##                      referring objects of the same type
+##  _   -     
+##  _ -   ,   
+##  _ - ,       
+##                      
+##  _   -      
+##               - - ,   
+##                             
+##                      
 ##
 sub _new
 {
@@ -103,7 +80,7 @@ sub _new
         $self = $class->_new_flat( class=>$arg{prop_class}, prop=>$prop[0]{name} );
     }
 
-    if( defined $self )
+    if( $self )
     {
         for( my $i=1; $i<@prop; $i++ )
         {
@@ -133,11 +110,11 @@ sub _new
 ## )
 ##
 ## class:
-##  The class that 'prop' property belongs to.
+##  ,    .
 ##
 ## prop:
-##  Direct property of the class.
-##  If this argument is omitted then 'id' is assumed.
+##    
+##     ,   id
 ##
 sub _new_flat
 {
@@ -158,7 +135,7 @@ sub _new_flat
     }
     else
     {
-        croak "'$arg{prop}' is neither a property of '$arg{class}' nor described in '$class' or parents"
+        croak "Class '$arg{class}' has no property named '$arg{prop}'";
     }
 
     return $self;
@@ -269,9 +246,10 @@ sub _sql_str
 ## METHODS
 ##
 
-## use: $metaprop->_expand( STRING );
+## use: $node->_expand( STRING );
 ##
-## Extends $metaprop meta-property to be 'prop' meta-property of $metaprop.
+##   prop  ,
+##      .
 ##
 sub _expand
 {
@@ -343,7 +321,6 @@ sub _rev_expand
     my $self      = shift;
     my $rev_class = shift;
     my $rev_prop  = shift;
-    my $cond      = shift;
 
     if( !$self->{prop_ref_class} )
     {
@@ -355,7 +332,7 @@ sub _rev_expand
     }
     else
     {
-        $self->_arb_expand( 'id' => $rev_class, $rev_prop, $cond );
+        $self->_arb_expand( 'id' => $rev_class, $rev_prop );
     }
 }
 
@@ -404,12 +381,13 @@ sub _arb_expand
 
 ## use: @prop = $prop->_parse_prop_str( str=>STRING );
 ##
-## Each element of resulting array is hash, containing fields:
+##      ,
+##   :
 ##
-##   type:  '>' - direct or '<' - reverse property
-##   name:  name of the property
-##   class: (only makes sence for reverse properties) referring class
-##   alias: (only makes sence for reverse properties) alias
+##   type:  '>' -   '<' -  
+##   name:   
+##   class: (   )  
+##   alias: (   ) 
 ##
 sub _parse_prop_str
 {
