@@ -1,40 +1,29 @@
 #
-# DESCRIPTION
+# DESCRIPTION:
 #   PerlORM - Object relational mapper (ORM) for Perl. PerlORM is Perl
 #   library that implements object-relational mapping. Its features are
 #   much similar to those of Java's Hibernate library, but interface is
 #   much different and easier to use.
 #
-# AUTHOR
+# AUTHOR:
 #   Alexey V. Akimov <akimov_alexey@sourceforge.net>
 #
 # COPYRIGHT
-#   Copyright (C) 2005-2006 Alexey V. Akimov
+#   Copyright (c) 2005 Alexey V. Akimov. All rights reserved.
 #
-#   This library is free software; you can redistribute it and/or
-#   modify it under the terms of the GNU Lesser General Public
-#   License as published by the Free Software Foundation; either
-#   version 2.1 of the License, or (at your option) any later version.
-#   
-#   This library is distributed in the hope that it will be useful,
-#   but WITHOUT ANY WARRANTY; without even the implied warranty of
-#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#   Lesser General Public License for more details.
-#   
-#   You should have received a copy of the GNU Lesser General Public
-#   License along with this library; if not, write to the Free Software
-#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-#
-
-# Instance of the class represents tree data structure,
-# describing links between DB tables in joins.
 
 package ORM::Tjoin;
 
 use Carp;
 use ORM::TjoinNull;
 
-$VERSION=0.81;
+##
+##      
+## ,       .
+##      .
+##
+
+$VERSION=0.8;
 
 ##
 ## CLASS METHODS
@@ -57,7 +46,7 @@ sub new
 
     if( ! exists $arg{class} )
     {
-        $self = ORM::TjoinNull->new( null_class=>$arg{null_class} );
+        $self = ORM::TjoinNull->new;
     }
     elsif( !UNIVERSAL::isa( $arg{class}, 'ORM' ) || $arg{class}->_is_initial )
     {
@@ -106,7 +95,7 @@ sub new
 
 sub copy
 {
-    # Must copy:
+    # Should copy:
     #
     # class
     # left_prop
@@ -149,7 +138,6 @@ sub copy
 ##
 
 sub class       { $_[0]->{class}; }
-sub null_class  { $_[0]->{class}; }
 sub fingerprint { $_[0]->{fingerprint}; }
 
 sub sql_cond_str
